<?php

defined('ABSPATH') || exit;

class SCB_Options_Page {
	public static function init(){
		add_action('admin_init', array(__CLASS__, 'register_options'));
		add_action('admin_menu', array(__CLASS__, 'register_page'));
		add_action('admin_enqueue_scripts', array(__CLASS__, 'register_scripts'));
	}

	public static function register_page(){
		if( !current_user_can('manage_options') ){ return; }
		
		add_management_page(
			__('Simple Country Block configuration', 'simplecountryblock'),
			__('Block by country', 'simplecountryblock'),
			'manage_options',
			SCB_Helpers::SECTION_NAME,
			array(__CLASS__, 'print_page_html')
		);
	}

	public static function register_scripts( string $current_page ){
		if( !current_user_can('manage_options') ){ return; }

		if( self::ends_with(SCB_Helpers::SECTION_NAME, $current_page) ){
			wp_enqueue_script('jquery');

			wp_enqueue_style('scb-styles', plugins_url('styles/styles.css', __FILE__), array(), '0.1.0');
			wp_enqueue_style('country-flags', plugins_url('styles/flags.min.css', __FILE__), array(), '0.1.0');
		}
	}

	public static function print_page_html(){
		?>
		<h1><?php echo get_admin_page_title(); ?></h1>

		<div class="scb-options-container">
			<form action="options.php" method="POST">
				<?php
					settings_fields(SCB_Helpers::SECTION_NAME);
					do_settings_sections('scb_options_main_section');
					submit_button();
				?>
			</form>

			<div class="scb-notices-container">
				<p><?php printf(__('This product includes GeoLite2 data created by MaxMind, available from %s.', 'simplecountryblock'), '<a href="https://www.maxmind.com">https://www.maxmind.com</a>') ?></p>

				<p><?php printf(__('Flags from CSS Flag Sprite generator (%s)', 'simplecountryblock'), '<a href="https://www.flag-sprites.com/">https://www.flag-sprites.com/</a>') ?></p>
			</div>
		</div>

		<?php
	}

	public static function register_options(){
		$register_arguments = array(
			'sanitize_callback'	=> array(__CLASS__, 'options_filter'),
			'default'			=> array(),
		);
		
		register_setting(SCB_Helpers::SECTION_NAME, SCB_Helpers::SECTION_NAME, $register_arguments);
		
		add_settings_section(
			'scb_options',
			__('Block dynamic content by country', 'simplecountryblock'),
			'',
			'scb_options_main_section'
		);
		
		add_settings_field(
			'scb_options_checklist',
			__('Select what countries to block', 'simplecountryblock'),
			array(__CLASS__, 'field_show_countries'),
			'scb_options_main_section',
			'scb_options',
		);
	}

	public static function field_show_countries(){
		$admin_details = new SCB_Visitor_Info($_SERVER['REMOTE_ADDR']);

		?>
		<p><?php _e('The system has detected this as your country of origin:', 'simplecountryblock'); ?></p>
		
		<div class="scb-admin-country-container">
			<?php echo self::get_country_html($admin_details->get_country_code()); ?>
		</div>

		<p><?php _e('Be aware that if you block your country, the system may lock out you from your website.', 'simplecountryblock'); ?></p>

		<p><?php _e('Please select the countries you want to block from accessing the dynamic content of your website.', 'simplecountryblock'); ?>
		<em><?php _e('For your convenience, the already selected countries are listed first.', 'simplecountryblock'); ?></em></p>

		<script>
			jQuery(document).ready(function(){
				jQuery('.scb-country-selection .scb-checkbox-container label').click(function(){
					var is_checked = jQuery(this).find(':checked');

					if( 0 < is_checked.length ){
						jQuery(this).parent().addClass('checked');
					} else {
						jQuery(this).parent().removeClass('checked');
					}
				});
			});
		</script>

		<?php
			/**
			 * Fires right before the country list is prepared and displayed in the options page.
			 */
			do_action('scb_before_country_list');
		?>

		<div class="scb-country-selection">
			<?php

			$selected_countries = array();
			$other_countries = array();

			$country_list = SCB_Helpers::get_country_list();
			foreach($country_list as $country_code => $not_used){
				$checked = SCB_Helpers::is_blocked_country($country_code) ? 'checked' : '';

				$checkbox_html = '
					<div class="scb-checkbox-container '. $checked .'">
					<label>
						<input
							type="checkbox"
							name="'. SCB_Helpers::SECTION_NAME .'[]"
							value="'. $country_code .'"
							'. $checked .'
						>
						<div class="scb-country-html">
							'. self::get_country_html($country_code) .'
						</div>
					</label>
					</div>
				';

				if( $checked ){
					$selected_countries[] = $checkbox_html;
				} else {
					$other_countries[] = $checkbox_html;
				}
			}

			echo implode("\n", $selected_countries);
			echo implode("\n", $other_countries);

			?>
		</div>

		<?php
			/**
			 * Fires right after the list of countries have been printed, but before the footer.
			 */
			do_action('scb_after_country_list');
		?>

		<?php
	}

	public static function options_filter( array $new_settings = NULL ){
		if( NULL === $new_settings || empty($new_settings) ){
			return array();
		}

		$country_list = SCB_Helpers::get_country_list();

		$safe_codes = array();
		foreach($new_settings as $current_country){
			if( isset($country_list[$current_country]) ){
				$safe_codes[] = $current_country;
			}
		}

		return $safe_codes;
	}

	private static function get_country_html( string $country_code ){
		$country_list = SCB_Helpers::get_country_list();

		$country_name = __('Unknown country', 'simplecountryblock');
		if( isset($country_list[$country_code]) ){
			$country_name = $country_list[$country_code];
		}

		return '
				<div class="scb-country-container">
					<div class="scb-flag flag flag-' . esc_attr(strtolower($country_code)) . '"></div>' .
					esc_html($country_name) .
				'</div>'
			;
	}

	private static function ends_with( string $search_word, string $string_to_search ){
		$length = strlen($search_word);
		if (0 === $length) {
			return TRUE;
		}

		return (substr($string_to_search, -$length) === $search_word);
	}
}

SCB_Options_Page::init();